from __future__ import annotations
from dataclasses import dataclass
from datetime import date, datetime, timedelta
from typing import Optional
import config


@dataclass
class Issue:
    summary: str
    description: str
    updated: date
    status: str
    miror: Optional[Issue] = None

    def delete(self):
        pass

    def get(self):
        pass

    def all(self):
        pass

    def update(self):
        pass

    def synchroniser_status_miroir_avec_status(self):
        if self.miror:
            print(self.miror.status)
            self.miror.status = config.status_dict_S2_to_S1[self.status]
            self.miror.update()
            return True
        return False

    @classmethod
    def convertirS_enS_(cls, issue):
        return cls(summary=issue.summary, description=issue.description, status=issue.status, updated=datetime.now())

    @staticmethod
    def convertir_en_format_date(date_to_convert):
        if isinstance(date_to_convert, datetime):
            date_to_convert = date_to_convert.strftime('%Y-%m-%d %H:%M:%S.%f')
        date = datetime.strptime(date_to_convert, '%Y-%m-%d %H:%M:%S.%f')
        offset = date.utcoffset()
        date_formattee = date + (offset if offset else timedelta(hours=0))
        return date_formattee

    @staticmethod
    def convertir_str_date_en_datetime(date_to_convert):
        date_without_timezone = date_to_convert[:-5]
        date_without_microseconds = datetime.strptime(date_without_timezone, '%Y-%m-%dT%H:%M:%S.%f')
        microseconds = int(date_to_convert[-5:])
        return date_without_microseconds.replace(microsecond=microseconds)

    @classmethod
    def all_attribut_id_et_updated(cls):
        issues = cls.all_filtre_id_et_updated()
        issues_list = []
        for issue in issues:
            issues_list.append(cls(summary="", description="", updated=issue.updated, status=""))
        return issues_list

    @staticmethod
    def date_S1_entre_date_derniere_synchro_et_date_S2(date_S1, date_S2):
        last_synchronized_date = Issue.convertir_str_date_en_datetime(config.last_synchronized_data_in_S2)
        date_S1_format_datetime = Issue.convertir_en_format_date(date_S1)
        date_S2_format_datetime = Issue.convertir_str_date_en_datetime(date_S2)

        return last_synchronized_date <= date_S1_format_datetime < date_S2_format_datetime

    def synchronisation_summary(self):
        if self.miror:
            print(self.miror.summary)
            self.summary = self.miror.summary
            self.update()
            return True
        return False

    def synchronisation_description(self):
        if self.miror:
            self.description = self.miror.description
            self.update()
            return True
        return False

    def synchronisation_status(self):
        if self.miror:
            try:
                self.change_issue_status(config.status_dict_S1_to_S2[self.miror.status])
                return True
            except Exception as e:
                try:
                    self.change_issue_status(config.status_dict_S2_to_S1[self.miror.status])
                    return True
                except Exception as e:
                    return False

    def synchroniser_status_avec_status_miroir(self):
        if self.miror:
            print(self.miror)
            self.change_issue_status(config.status_dict_S1_to_S2[self.miror.status])
            return True
        return False

    def synchroniser_status_miroir_status(self):
        if self.miror:
            print(self.miror.status)
            self.miror.status = config.status_dict_S2_to_S1[self.status]
            self.miror.update()
            return True
        return False

    def synchronisation_summary_from_miror(self):
        self.summary = self.miror.summary
        # Il faut juste qu'il update summary et non pas description aussi et tout !
        self.update()

    def synchronisation_summary_to_miror(self):
        self.miror.summary = self.summary
        self.update()

    def synchronisation_description_from_miror(self):
        self.description = self.miror.description
        self.update()

    def synchronisation_description_to_miror(self):
        self.miror.description = self.description
        self.update()



import json
import requests
import config
from domains.issue import Issue
from implementations.implementation_issue_S1 import IssueImplementationS1


class IssueImplementationS2(Issue):
    key: str

    def __init__(self, key=None, summary=None, description=None, updated=None, status=None):
        Issue.__init__(self, summary, description, updated, status)
        self.key = key
        if self.key is not None :
            self.miror = self.get_miror()
        if self.miror is not None:
            self.miror.miror = self

    def format_issue_into_json(self):
        issue = {
            "fields": {
                "description": {
                    "content": [
                        {
                            "content": [
                                {
                                    "text": self.description,
                                    "type": "text"
                                }
                            ],
                            "type": "paragraph"
                        }
                    ],
                    "type": "doc",
                    "version": 1
                },
                "issuetype": {
                    "id": config.key_issue_type
                },
                "project": {
                    "id": config.project_key
                },
                "summary": self.summary
            },
            "update": {},
        }
        if self.miror is not None:
            issue['fields'][config.s1_id_in_jira] = str(self.miror.id)
        return json.dumps(issue)

    @staticmethod
    def convertir_json_en_issue(issue_json):
        try:
            issue_key = issue_json.get('key', None)
            issue_summary = issue_json['fields'].get('summary', None)
            description_data = issue_json['fields'].get('description', None)

            description_text = None
            if description_data is not None and 'content' in description_data:
                content_list = description_data['content']

                if content_list and content_list[0].get('content'):
                    description_text = content_list[0]['content'][0].get('text', None)

            issue_updated = issue_json['fields'].get('updated', None)
            status = issue_json['fields']['status']['name']
            return IssueImplementationS2(key=issue_key, summary=issue_summary, description=description_text,
                                         updated=issue_updated,
                                         status=status)
        except Exception as e:
            print(e)
            return None

    def get_miror(self):
        response = requests.request("GET", config.jira_url_ticket + self.key, headers=config.headers, auth=config.auth)
        if response.status_code == 200:
            mirror_id = response.json()['fields'].get(config.s1_id_in_jira, None)
            miror = IssueImplementationS1.find_by_id(mirror_id)
            return miror

    def get(self):
        try:
            response = requests.request("GET", config.jira_url_ticket + self.key, headers=config.headers, auth=config.auth)
            json_data = response.json()
            issue = IssueImplementationS2.convertir_json_en_issue(json_data)
            if issue is not None:
                return IssueImplementationS2(key=issue.key, summary=issue.summary, description=issue.description,
                                             updated=issue.updated, status=issue.status)
        except Exception as e:
            return e

    @staticmethod
    def find_by_id(key):
        try:
            response = requests.request("GET", config.jira_url_ticket + key, headers=config.headers, auth=config.auth)
            json_data = response.json()
            issue = IssueImplementationS2.convertir_json_en_issue(json_data)
            if issue is not None:
                return IssueImplementationS2(key=issue.key, summary=issue.summary, description=issue.description,
                                             updated=issue.updated, status=issue.status)
        except Exception as e:
            return e

    @staticmethod
    def find_by(jql_query):
        payload = {"jql": jql_query}
        response = requests.get(config.jira_url_all, params=payload, headers=config.headers, auth=config.auth)
        if response.status_code == 200:
            return response.json().get("issues", [])
        else:
            print(f"Failed to execute search query. Status code: {response.status_code}")
            return []

    @staticmethod
    def delete_all(jql_query):
        payload = {"jql": jql_query}
        response = requests.get(config.jira_url_all, params=payload, headers=config.headers, auth=config.auth)
        if response.status_code == 200:
            issues = response.json().get("issues", [])
            for issue in issues:
                IssueImplementationS2.delete(issue["key"])
        else:
            return f"Failed to execute search query. Status code: {response.status_code}"

    def change_status_name(self):
        data = {"status": {"name": "En cours"}}
        headers = {'Content-Type': 'application/json'}
        response = requests.post(config.jira_url_ticket + self.key, json=data, headers=headers, auth=config.auth)
        print(response)

    @staticmethod
    def all():
        response = requests.get(config.jira_url_all, headers=config.headers, auth=config.auth)
        issues = response.json()["issues"]
        issues_list = []
        for issue in issues:
            issues_list.append(IssueImplementationS2.convertir_json_en_issue(issue))

        return issues_list

    @staticmethod
    def all_filtre_id_et_updated():
        response = requests.get(config.jira_url_all, headers=config.headers, auth=config.auth)
        issues = response.json()["issues"]
        issues_list = []
        for issue in issues:
            issue_key = issue.get('key', None)
            issue_updated = issue['fields'].get('updated', None)
            issues_list.append(
                IssueImplementationS2(key=issue_key, summary="", description="", updated=issue_updated, status=""))
        return issues_list

    def save(self):
        response = requests.request("POST", config.jira_url_ticket, data=self.format_issue_into_json(), headers=config.headers,
                                    auth=config.auth)
        if response.status_code == 201:
            return (json.loads(response.text)).get("key")
        else:
            return f"Erreur lors de la création du ticket. Code d'erreur : {response.status_code}"

    def delete(self):
        response = requests.request("DELETE", config.jira_url_ticket + self.key, auth=config.auth)

        if response.status_code == 204:
            print(f"Suppression effectuée avec succées")
        else:
            print(f"Erreur lors de la suppression du ticket. Code d'erreur : {response.status_code}")

    def update(self):
        response = requests.request("PUT", config.jira_url_ticket + self.key, data=self.format_issue_into_json(),
                                    headers=config.headers, auth=config.auth)
        return response.text

    def change_issue_status(self, new_status):
        transition_url = config.jira_url_ticket + self.key + "/transitions"
        response = requests.get(transition_url, auth=config.auth)
        transitions = response.json()['transitions']
        transition_id = None
        for transition in transitions:
            if transition['name'] == new_status:
                transition_id = transition['id']
                break
        if not transition_id:
            return None

        data = {
            "transition": {"id": transition_id}
        }
        headers = {'Content-Type': 'application/json'}

        response = requests.post(transition_url, json=data, headers=headers, auth=config.auth)

        if response.status_code == 204:
            return transition_id
        else:
            print("Erreur lors du changement de statut:", response.text)
            return None

    @staticmethod
    def supprimer_tickets_entre_a_and_b(a, b):
        for i in range(a, b):
            key = "KAN-" + str(i)
            issue_to_delete = IssueImplementationS2(key=key, summary="", description="", updated="", status="")
            issue_to_delete.delete()


if __name__ == '__main__':
    IssueImplementationS2.supprimer_tickets_entre_a_and_b(1045, 1190)
